@@ -43,7 +43,7 @@ static int expunged = 0;
 static unsigned int *unseen_messages;
 
 /* for "IMAP> EXPUNGE" */
-static int recentcount_ok = 0;
+static int actual_deletions = 0;
 
 /* for "IMAP> IDLE" */
 static int saved_timeout = 0;
@@ -84,7 +84,7 @@ static int imap_ok(int sock, char *argbuf)
 		 * count, and allocate a malloc area that would overlap
 		 * a portion of the stack.
 		 */
-		if (count > INT_MAX/sizeof(int))
+		if ((unsigned)count > INT_MAX/sizeof(int))
 		{
 		    report(stderr, GT_("bogus message count!"));
 		    return(PS_PROTOCOL);
@@ -114,7 +114,6 @@ static int imap_ok(int sock, char *argbuf)
 	    }
 	    else if (strstr(buf, " RECENT"))
 	    {
-		recentcount_ok = 1;
 		recentcount = atoi(buf+2);
 	    }
 	    else if (strstr(buf, " EXPUNGE"))
@@ -122,9 +121,18 @@ static int imap_ok(int sock, char *argbuf)
 		/* the response "* 10 EXPUNGE" means that the currently
 		 * tenth (i.e. only one) message has been deleted */
 		if (atoi(buf+2) > 0)
-		    count--;
-		if (count < 0)
-		    count = 0;
+		{
+		    if (count > 0)
+			count--;
+		    /* Some servers do not report RECENT after an EXPUNGE.
+		     * For such servers, assume that the mail being
+		     * expunged is a recent one. For other servers, we
+		     * should get an updated RECENT report later and this
+		     * assumption will have no effect. */
+		    if (recentcount > 0)
+			recentcount--;
+		    actual_deletions++;
+		}
 	    }
 	    else if (strstr(buf, " PREAUTH"))
 	    {
@@ -225,7 +233,7 @@ static int do_imap_ntlm(int sock, struct query *ctl)
 	dumpSmbNtlmAuthRequest(stdout, &request);
 
     memset(msgbuf,0,sizeof msgbuf);
-    to64frombits (msgbuf, (unsigned char*)&request, SmbLength(&request));
+    to64frombits (msgbuf, &request, SmbLength(&request));
   
     if (outlevel >= O_MONITOR)
 	report(stdout, "IMAP> %s\n", msgbuf);
@@ -236,7 +244,7 @@ static int do_imap_ntlm(int sock, struct query *ctl)
     if ((gen_recv(sock, msgbuf, sizeof msgbuf)))
 	return result;
   
-    len = from64tobits ((char*)&challenge, msgbuf, sizeof(challenge));
+    len = from64tobits (&challenge, msgbuf, sizeof(challenge));
     
     if (outlevel >= O_DEBUG)
 	dumpSmbNtlmAuthChallenge(stdout, &challenge);
@@ -247,7 +255,7 @@ static int do_imap_ntlm(int sock, struct query *ctl)
 	dumpSmbNtlmAuthResponse(stdout, &response);
   
     memset(msgbuf,0,sizeof msgbuf);
-    to64frombits (msgbuf, (unsigned char*)&response, SmbLength(&response));
+    to64frombits (msgbuf, &response, SmbLength(&response));
 
     if (outlevel >= O_MONITOR)
 	report(stdout, "IMAP> %s\n", msgbuf);
@@ -265,10 +273,10 @@ static int do_imap_ntlm(int sock, struct query *ctl)
 }
 #endif /* NTLM */
 
-static int imap_canonicalize(char *result, char *raw, int maxlen)
+static void imap_canonicalize(char *result, char *raw, size_t maxlen)
 /* encode an IMAP password as per RFC1730's quoting conventions */
 {
-    int i, j;
+    size_t i, j;
 
     j = 0;
     for (i = 0; i < strlen(raw) && i < maxlen; i++)
@@ -278,8 +286,6 @@ static int imap_canonicalize(char *result, char *raw, int maxlen)
 	result[j++] = raw[i];
     }
     result[j] = '\0';
-
-    return(i);
 }
 
 static void capa_probe(int sock, struct query *ctl)
@@ -345,6 +351,7 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
     flag did_stls = FALSE;
 #endif /* SSL_ENABLE */
 
+    (void)greeting;
     /*
      * Assumption: expunges are cheap, so we want to do them
      * after every message unless user said otherwise.
@@ -577,15 +584,26 @@ static int internal_expunge(int sock)
 {
     int	ok;
 
-    recentcount_ok = 0;
+    actual_deletions = 0;
 
     if ((ok = gen_transact(sock, "EXPUNGE")))
 	return(ok);
 
-    /* some servers do not report RECENT after an EXPUNGE. in this case, 
-     * the previous value of recentcount is just ignored. */
-    if (!recentcount_ok)
-        recentcount = 0;
+    /* if there is a mismatch between the number of mails which should
+     * have been expunged and the number of mails actually expunged,
+     * another email client may be deleting mails. Quit here,
+     * otherwise fetchmail gets out-of-sync with the imap server,
+     * reports the wrong size to the SMTP server on MAIL FROM: and
+     * triggers a "message ... was not the expected length" error on
+     * every subsequent mail */
+    if (deletions > 0 && deletions != actual_deletions)
+    {
+	report(stderr,
+		GT_("mail expunge mismatch (%d actual != %d expected)\n"),
+		actual_deletions, deletions);
+	deletions = 0;
+	return(PS_ERROR);
+    }
 
     expunged += deletions;
     deletions = 0;
@@ -602,7 +620,6 @@ static int imap_idle(int sock)
 {
     int ok;
 
-    stage = STAGE_IDLE;
     saved_timeout = mytimeout;
 
     if (has_idle) {
@@ -610,6 +627,7 @@ static int imap_idle(int sock)
 	 * at least every 28 minutes:
 	 * (the server may have an inactivity timeout) */
 	mytimeout = 1680; /* 28 min */
+	stage = STAGE_IDLE;
 	/* enter IDLE mode */
 	ok = gen_transact(sock, "IDLE");
 
@@ -618,37 +636,43 @@ static int imap_idle(int sock)
 	    SockWrite(sock, "DONE\r\n", 6);
 	    if (outlevel >= O_MONITOR)
 		report(stdout, "IMAP> DONE\n");
-	} else
-	    /* not idle timeout */
-	    return ok;
+	    /* reset stage and timeout here: we are not idling any more */
+	    mytimeout = saved_timeout;
+	    stage = STAGE_FETCH;
+	    /* get OK IDLE message */
+	    ok = imap_ok(sock, NULL);
+	}
     } else {  /* no idle support, fake it */
-	/* when faking an idle, we can't assume the server will
-	 * send us the new messages out of the blue (RFC2060);
-	 * this timeout is potentially the delay before we notice
-	 * new mail (can be small since NOOP checking is cheap) */
-	mytimeout = 28;
+	/* Note: stage and timeout have not been changed here as NOOP
+	 * does not idle */
 	ok = gen_transact(sock, "NOOP");
-	/* if there's an error (not likely) or we just found mail (stage 
-	 * has changed, timeout has also been restored), we're done */
-	if (ok != 0 || stage != STAGE_IDLE)
-	    return(ok);
 
-	/* wait (briefly) for an unsolicited status update */
-	ok = imap_ok(sock, NULL);
-	/* again, this is new mail or an error */
-	if (ok != PS_IDLETIMEOUT)
-	    return(ok);
+	/* no error, but no new mail either */
+	if (ok == PS_SUCCESS && recentcount == 0)
+	{
+	    /* There are some servers who do send new mail
+	     * notification out of the blue. This is in compliance
+	     * with RFC 2060 section 5.3. Wait for that with a low
+	     * timeout */
+	    mytimeout = 28;
+	    stage = STAGE_IDLE;
+	    /* We are waiting for notification; no tag needed */
+	    tag[0] = '\0';
+	    /* wait (briefly) for an unsolicited status update */
+	    ok = imap_ok(sock, NULL);
+	    if (ok == PS_IDLETIMEOUT) {
+		/* no notification came; ok */
+		ok = PS_SUCCESS;
+	    }
+	}
     }
 
     /* restore normal timeout value */
+    set_timeout(0);
     mytimeout = saved_timeout;
     stage = STAGE_FETCH;
 
-    /* get OK IDLE message */
-    if (has_idle)
-        return imap_ok(sock, NULL);
-
-    return PS_SUCCESS;
+    return(ok);
 }
 
 static int imap_getrange(int sock, 
@@ -712,16 +736,6 @@ static int imap_getrange(int sock,
 				    "%d messages waiting after first poll\n",
 				    count), count);
 
-	/* no messages?  then we may need to idle until we get some */
-	while (count == 0 && do_idle) {
-	    ok = imap_idle(sock);
-	    if (ok)
-	    {
-		report(stderr, GT_("re-poll failed\n"));
-		return(ok);
-	    }
-	}
-
 	/*
 	 * We should have an expunge here to
 	 * a) avoid fetching deleted mails during 'fetchall'
@@ -740,6 +754,23 @@ static int imap_getrange(int sock,
 					"%d messages waiting after expunge\n",
 					count), count);
 	}
+
+	if (count == 0 && do_idle)
+	{
+	    /* no messages?  then we may need to idle until we get some */
+	    while (count == 0) {
+		ok = imap_idle(sock);
+		if (ok)
+		{
+		    report(stderr, GT_("re-poll failed\n"));
+		    return(ok);
+		}
+	    }
+	    if (outlevel >= O_DEBUG)
+		report(stdout, ngettext("%d message waiting after re-poll\n",
+					"%d messages waiting after re-poll\n",
+					count), count);
+	}
     }
 
     *countp = count;
@@ -751,7 +782,7 @@ static int imap_getrange(int sock,
     {
 	if (unseen_messages)
 	    free(unseen_messages);
-	unseen_messages = xmalloc(count * sizeof(unsigned int));
+	unseen_messages = (unsigned int *)xmalloc(count * sizeof(unsigned int));
 	memset(unseen_messages, 0, count * sizeof(unsigned int));
 	unseen = 0;
 
@@ -781,17 +812,15 @@ static int imap_getrange(int sock,
 			cp++;
 		    if (*cp) 
 		    {
-			unsigned int um;
-			/*
-			 * Message numbers are between 1 and 2^32 inclusive,
-			 * so unsigned int is large enough.
-			 */
-			um=(unsigned int)strtol(cp,&ep,10);
-			if (um <= count)
+			unsigned long um;
+
+			errno = 0;
+			um = strtoul(cp,&ep,10);
+			if (errno == 0 && um <= UINT_MAX && um <= (unsigned)count)
 			{
 			    unseen_messages[unseen++] = um;
 			    if (outlevel >= O_DEBUG)
-				report(stdout, GT_("%u is unseen\n"), um);
+				report(stdout, GT_("%lu is unseen\n"), um);
 			    if (startcount > um)
 				startcount = um;
 			}
@@ -866,8 +895,8 @@ static int imap_getpartialsizes(int sock, int first, int last, int *sizes)
 	return(PS_SUCCESS);
     for (;;)
     {
-	unsigned int num, size;
-	int ok;
+	unsigned int size;
+	int ok, num;
 	char *cp;
 
 	if ((ok = gen_recv(sock, buf, sizeof(buf))))
@@ -880,7 +909,7 @@ static int imap_getpartialsizes(int sock, int first, int last, int *sizes)
 	    ;
 	else if (strstr(buf, "OK") || strstr(buf, "NO"))
 	    break;
-	else if (sscanf(buf, "* %u FETCH (RFC822.SIZE %u)", &num, &size) == 2
+	else if (sscanf(buf, "* %d FETCH (RFC822.SIZE %u)", &num, &size) == 2
 	/* some servers (like mail.internode.on.net bld-mail04) return UID information here
 	 *
 	 * IMAP> A0005 FETCH 1 RFC822.SIZE
@@ -888,7 +917,7 @@ static int imap_getpartialsizes(int sock, int first, int last, int *sizes)
 	 * IMAP< A0005 OK FETCH completed
 	 *
 	 */
-		|| sscanf(buf, "* %u FETCH (UID %*s RFC822.SIZE %u)", &num, &size) == 2)
+		|| sscanf(buf, "* %d FETCH (UID %*s RFC822.SIZE %u)", &num, &size) == 2)
 	{
 	    if (num >= first && num <= last)
 		sizes[num - first] = size;
@@ -913,6 +942,8 @@ static int imap_is_old(int sock, struct query *ctl, int number)
     flag seen = TRUE;
     int i;
 
+    (void)sock;
+    (void)ctl;
     /* 
      * Expunges change the fetch numbers, but unseen_messages contains
      * indices from before any expungees were done.  So neither the
@@ -921,7 +952,7 @@ static int imap_is_old(int sock, struct query *ctl, int number)
 
     seen = TRUE;
     for (i = 0; i < unseen; i++)
-	if (unseen_messages[i] == number)
+	if (unseen_messages[i] == (unsigned)number)
 	{
 	    seen = FALSE;
 	    break;
@@ -944,6 +975,7 @@ static int imap_fetch_headers(int sock, struct query *ctl,int number,int *lenp)
     char buf [MSGBUFSIZE+1];
     int	num;
 
+    (void)ctl;
     /* expunges change the fetch numbers */
     number -= expunged;
 
@@ -1003,6 +1035,7 @@ static int imap_fetch_body(int sock, struct query *ctl, int number, int *lenp)
     char buf [MSGBUFSIZE+1], *cp;
     int	num;
 
+    (void)ctl;
     /* expunges change the fetch numbers */
     number -= expunged;
 
@@ -1071,12 +1104,13 @@ static int imap_fetch_body(int sock, struct query *ctl, int number, int *lenp)
     return(PS_SUCCESS);
 }
 
-static int imap_trail(int sock, struct query *ctl, int number, const char *tag)
+static int imap_trail(int sock, struct query *ctl, const char *tag)
 /* discard tail of FETCH response after reading message text */
 {
     /* expunges change the fetch numbers */
     /* number -= expunged; */
 
+    (void)ctl;
     for (;;)
     {
 	char buf[MSGBUFSIZE+1], *t;
@@ -1102,6 +1136,7 @@ static int imap_delete(int sock, struct query *ctl, int number)
 {
     int	ok;
 
+    (void)ctl;
     /* expunges change the fetch numbers */
     number -= expunged;
 
@@ -1131,7 +1166,10 @@ static int imap_delete(int sock, struct query *ctl, int number)
      * the next session.
      */
     if (NUM_NONZERO(expunge_period) && (deletions % expunge_period) == 0)
-	internal_expunge(sock);
+    {
+	if ((ok = internal_expunge(sock)))
+	    return(ok);
+    }
 
     return(PS_SUCCESS);
 }
@@ -1139,6 +1177,7 @@ static int imap_delete(int sock, struct query *ctl, int number)
 static int imap_mark_seen(int sock, struct query *ctl, int number)
 /* mark the given message as seen */
 {
+    (void)ctl;
     return(gen_transact(sock,
 	imap_version == IMAP4
 	? "STORE %d +FLAGS.SILENT (\\Seen)"
@@ -1149,6 +1188,7 @@ static int imap_mark_seen(int sock, struct query *ctl, int number)
 static int imap_end_mailbox_poll(int sock, struct query *ctl)
 /* cleanup mailbox before we idle or switch to another one */
 {
+    (void)ctl;
     if (deletions)
 	internal_expunge(sock);
     return(PS_SUCCESS);
@@ -1157,6 +1197,7 @@ static int imap_end_mailbox_poll(int sock, struct query *ctl)
 static int imap_logout(int sock, struct query *ctl)
 /* send logout command */
 {
+    (void)ctl;
     /* if any un-expunged deletions remain, ship an expunge now */
     if (deletions)
 	internal_expunge(sock);
