@@ -441,7 +441,7 @@ int SockRead(int sock, char *buf, int len)
 		} else if ((newline = memchr(bp, '\n', n)) != NULL)
 			n = newline - bp + 1;
 		/* Matthias Andree: SSL_read can return 0, in that case
-		 * we must cal SSL_get_error to figure if there was
+		 * we must call SSL_get_error to figure if there was
 		 * an error or just a "no data" condition */
 		if ((n = SSL_read(ssl, bp, n)) <= 0) {
 			if ((n = SSL_get_error(ssl, n))) {
@@ -706,10 +706,10 @@ static int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 		}
 		/* Print the finger print. Note that on errors, we might print it more than once
 		 * normally; we kluge around that by using a global variable. */
-		if (_check_fp) {
+		if (_check_fp == 1) {
 			unsigned dp;
 
-			_check_fp = 0;
+			_check_fp = -1;
 			digest_tp = EVP_md5();
 			if (digest_tp == NULL) {
 				report(stderr, GT_("EVP_md5() failed!\n"));
@@ -736,23 +736,22 @@ static int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 				    if (outlevel > O_NORMAL)
 					report(stdout, GT_("%s fingerprints match.\n"), _server_label);
 				} else {
-				    if (outlevel > O_SILENT)
-					report(stderr, GT_("%s fingerprints do not match!\n"), _server_label);
+				    report(stderr, GT_("%s fingerprints do not match!\n"), _server_label);
 				    return (0);
 				}
-			}
-		}
-	}
+			} /* if (_check_digest != NULL) */
+		} /* if (_check_fp) */
+	} /* if (depth == 0 && !_depth0ck) */
 
-	if (err != X509_V_OK && err != _prev_err) {
-        	_prev_err = err;
+	if (err != X509_V_OK && err != _prev_err && !(_check_fp != 0 && _check_digest && !strict)) {
+		_prev_err = err;
 		report(stderr, GT_("Server certificate verification error: %s\n"), X509_verify_cert_error_string(err));
 		/* We gave the error code, but maybe we can add some more details for debugging */
 		switch (err) {
 		case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
 			X509_NAME_oneline(issuer, buf, sizeof(buf));
 			buf[sizeof(buf) - 1] = '\0';
-			report(stderr, GT_("unknown issuer (first %d characters): %s\n"), sizeof(buf)-1, buf);
+			report(stderr, GT_("unknown issuer (first %d characters): %s\n"), (int)(sizeof(buf)-1), buf);
 			break;
 		}
 	}
