@@ -1164,7 +1164,7 @@ int readheaders(int sock,
 		    saddr, sizeof(saddr), NULL, 0,
 		    NI_NUMERICHOST);
 	    if (e)
-		snprintf(saddr, sizeof(saddr), "(%-.*s)", sizeof(saddr) - 3, gai_strerror(e));
+		snprintf(saddr, sizeof(saddr), "(%-.*s)", (int)(sizeof(saddr) - 3), gai_strerror(e));
 	    snprintf(buf, sizeof(buf),
 		    "Received: from %s [%s]\r\n", 
 		    ctl->server.truename, saddr);
@@ -1190,6 +1190,10 @@ int readheaders(int sock,
 			" polling %s account %s",
 			ctl->server.pollname,
 			ctl->remotename);
+		if (ctl->folder)
+		    snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
+			    " folder %s",
+			    ctl->folder);
 	    }
 	    snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), ")\r\n");
 	    n = stuffline(ctl, buf);
